@@ -2,7 +2,7 @@ from collections import Counter, defaultdict
 from functools import reduce
 from itertools import chain
 from operator import mul
-from typing import Dict, Iterable, Iterator, List, Optional, Tuple, Type, TypeVar, cast
+from typing import Dict, Iterator, Optional, Tuple, cast
 
 from sympy import Expr, Function, var
 
@@ -14,12 +14,11 @@ from comb_spec_searcher import (
 )
 from comb_spec_searcher.exception import InvalidOperationError, StrategyDoesNotApply
 from comb_spec_searcher.typing import Objects, Terms
-from permuta import Perm
+from permuta import Av, Perm
 from permuta.permutils import (
     is_insertion_encodable_maximum,
     is_insertion_encodable_rightmost,
 )
-from permuta.permutils.symmetry import all_symmetry_sets
 from tilings import GriddedPerm, Tiling
 from tilings.algorithms import locally_factorable_shift
 from tilings.algorithms.enumeration import (
@@ -31,13 +30,15 @@ from tilings.assumptions import ComponentAssumption
 from tilings.strategies import (
     FactorFactory,
     FactorInsertionFactory,
+    RemoveRequirementFactory,
     RequirementCorroborationFactory,
 )
 
+from .abstract import BasisAwareVerificationStrategy
+
 x = var("x")
 
 __all__ = [
-    "BasisAwareVerificationStrategy",
     "BasicVerificationStrategy",
     "OneByOneVerificationStrategy",
     "DatabaseVerificationStrategy",
@@ -49,75 +50,6 @@ __all__ = [
 
 
 TileScopeVerificationStrategy = VerificationStrategy[Tiling, GriddedPerm]
-BasisAwareVerificationStrategyType = TypeVar(
-    "BasisAwareVerificationStrategyType", bound="BasisAwareVerificationStrategy"
-)
-
-
-class BasisAwareVerificationStrategy(TileScopeVerificationStrategy):
-    """
-    A base class for a verification strategy that needs to know the basis the
-    Tilescope is currently running.
-    """
-
-    def __init__(
-        self,
-        basis: Optional[Iterable[Perm]] = None,
-        symmetry: bool = False,
-        ignore_parent: bool = True,
-    ):
-        self._basis = tuple(basis) if basis is not None else tuple()
-        self._symmetry = symmetry
-        assert all(
-            isinstance(p, Perm) for p in self._basis
-        ), "Element of the basis must be Perm"
-        if symmetry:
-            self.symmetries = frozenset(
-                frozenset(b) for b in all_symmetry_sets(self._basis)
-            )
-        else:
-            self.symmetries = frozenset([frozenset(self._basis)])
-        super().__init__(ignore_parent=ignore_parent)
-
-    def change_basis(
-        self: BasisAwareVerificationStrategyType, basis: Iterable[Perm], symmetry: bool
-    ) -> BasisAwareVerificationStrategyType:
-        """
-        Return a new version of the verification strategy with the given basis instead
-        of the current one.
-        """
-        basis = tuple(basis)
-        return self.__class__(basis, symmetry, self.ignore_parent)
-
-    @property
-    def basis(self) -> Tuple[Perm, ...]:
-        return self._basis
-
-    def to_jsonable(self) -> dict:
-        d: dict = super().to_jsonable()
-        d["basis"] = self._basis
-        d["symmetry"] = self._symmetry
-        return d
-
-    @classmethod
-    def from_dict(
-        cls: Type[BasisAwareVerificationStrategyType], d: dict
-    ) -> BasisAwareVerificationStrategyType:
-        if "basis" in d and d["basis"] is not None:
-            basis: Optional[List[Perm]] = [Perm(p) for p in d.pop("basis")]
-        else:
-            basis = d.pop("basis", None)
-        return cls(basis=basis, **d)
-
-    def __repr__(self) -> str:
-        args = ", ".join(
-            [
-                f"basis={self._basis}",
-                f"symmetry={self._symmetry}",
-                f"ignore_parent={self.ignore_parent}",
-            ]
-        )
-        return f"{self.__class__.__name__}({args})"
 
 
 class BasicVerificationStrategy(AtomStrategy):
@@ -246,11 +178,17 @@ class OneByOneVerificationStrategy(BasisAwareVerificationStrategy):
         )
 
     def verified(self, comb_class: Tiling) -> bool:
-        return comb_class.dimensions == (1, 1) and (
-            frozenset(ob.patt for ob in comb_class.obstructions) not in self.symmetries
-            or any(
-                isinstance(ass, ComponentAssumption) for ass in comb_class.assumptions
-            )
+        if not comb_class.dimensions == (1, 1):
+            return False
+        if not self.basis:
+            return True
+        tiling_class = Av([ob.patt for ob in comb_class.obstructions])
+        sym_classes = (Av(sym) for sym in self.symmetries)
+        is_strict_subclass = any(
+            tiling_class.is_subclass(cls) and cls != tiling_class for cls in sym_classes
+        )
+        return is_strict_subclass or any(
+            isinstance(ass, ComponentAssumption) for ass in comb_class.assumptions
         )
 
     def get_genf(
@@ -294,7 +232,9 @@ class OneByOneVerificationStrategy(BasisAwareVerificationStrategy):
         )
 
     def __str__(self) -> str:
-        return "one by one verification"
+        if not self.basis:
+            return "one by one verification"
+        return f"One by one subclass of {Av(self.basis)}"
 
 
 class DatabaseVerificationStrategy(TileScopeVerificationStrategy):
@@ -368,8 +308,29 @@ class LocallyFactorableVerificationStrategy(BasisAwareVerificationStrategy):
     verified tiling.
     """
 
+    def pack(self, comb_class: Tiling) -> StrategyPack:
+        if any(isinstance(ass, ComponentAssumption) for ass in comb_class.assumptions):
+            raise InvalidOperationError(
+                "Can't find generating function with component assumption."
+            )
+        return StrategyPack(
+            name="LocallyFactorable",
+            initial_strats=[FactorFactory(), RequirementCorroborationFactory()],
+            inferral_strats=[],
+            expansion_strats=[[FactorInsertionFactory()], [RemoveRequirementFactory()]],
+            ver_strats=[
+                BasicVerificationStrategy(),
+                OneByOneVerificationStrategy(
+                    basis=self._basis, symmetry=self._symmetry
+                ),
+                InsertionEncodingVerificationStrategy(),
+                MonotoneTreeVerificationStrategy(no_factors=True),
+                LocalVerificationStrategy(no_factors=True),
+            ],
+        )
+
     @staticmethod
-    def pack(comb_class: Tiling) -> StrategyPack:
+    def _pack_for_shift(comb_class: Tiling) -> StrategyPack:
         if any(isinstance(ass, ComponentAssumption) for ass in comb_class.assumptions):
             raise InvalidOperationError(
                 "Can't find generating function with component assumption."
@@ -419,8 +380,9 @@ class LocallyFactorableVerificationStrategy(BasisAwareVerificationStrategy):
         if self.verified(comb_class):
             if not self.basis:
                 return ()
+            pack = self._pack_for_shift(comb_class)
             sfs = locally_factorable_shift.shift_from_spec(
-                comb_class, self, self.symmetries
+                comb_class, pack, self.symmetries
             )
             if sfs is not None:
                 return (Tiling.from_perms(self.basis),)
@@ -436,8 +398,9 @@ class LocallyFactorableVerificationStrategy(BasisAwareVerificationStrategy):
                 raise StrategyDoesNotApply
         if not children:
             return ()
+        pack = self._pack_for_shift(comb_class)
         shift = locally_factorable_shift.shift_from_spec(
-            comb_class, self, self.symmetries
+            comb_class, pack, self.symmetries
         )
         assert shift is not None
         return (shift,)
@@ -488,7 +451,7 @@ class LocalVerificationStrategy(TileScopeVerificationStrategy):
     localized, i.e. in a single cell and the tiling is not 1x1.
     """
 
-    def __init__(self, ignore_parent: bool = True, no_factors: bool = False):
+    def __init__(self, ignore_parent: bool = False, no_factors: bool = False):
         self.no_factors = no_factors
         super().__init__(ignore_parent=ignore_parent)
 
@@ -579,7 +542,7 @@ class InsertionEncodingVerificationStrategy(TileScopeVerificationStrategy):
     Verify all n x 1 and 1 x n tilings that have a regular insertion encoding.
     """
 
-    def __init__(self, ignore_parent: bool = True):
+    def __init__(self, ignore_parent: bool = False):
         super().__init__(ignore_parent=ignore_parent)
 
     def pack(self, comb_class: Tiling) -> StrategyPack:
@@ -654,7 +617,7 @@ class MonotoneTreeVerificationStrategy(TileScopeVerificationStrategy):
     Verify all tiling that is a monotone tree.
     """
 
-    def __init__(self, ignore_parent: bool = True, no_factors: bool = True):
+    def __init__(self, ignore_parent: bool = False, no_factors: bool = True):
         self.no_factors = no_factors
         super().__init__(ignore_parent=ignore_parent)
 
